<?php
/**
 * @author Daniel J. Summers <daniel@bitbadger.solutions>
 * @license MIT
 */

declare(strict_types=1);

namespace MyPrayerJournal;

use Auth0\SDK\Auth0;
use Auth0\SDK\Exception\ConfigurationException;

/**
 * myPrayerJournal-specific authorization functions
 */
class Auth
{
    /** @var Auth0|null The Auth0 client to use for requests (initialized on first use) */
    private static ?Auth0 $auth0 = null;

    /**
     * Get the current Auth0 client
     *
     * @return Auth0 The current Auth0 client
     */
    public static function client(): Auth0
    {
        if (is_null(self::$auth0)) {
            self::$auth0 = new Auth0([
                'domain'       => $_ENV['AUTH0_DOMAIN'],
                'clientId'     => $_ENV['AUTH0_CLIENT_ID'],
                'clientSecret' => $_ENV['AUTH0_CLIENT_SECRET'],
                'cookieSecret' => $_ENV['AUTH0_COOKIE_SECRET']
            ]);
        }
        return self::$auth0;
    }

    /**
     * Get the logged on user information
     *
     * @return array|null The user information (null if no user is logged on)
     */
    public static function user(): ?array
    {
        return self::client()->getUser();
    }

    /**
     * Initiate a log on with Auth0
     *
     * @throws ConfigurationException If the Auth0 client is not configured correctly
     */
    public static function logOn(): never
    {
        $params = match (true) {
            $_SERVER['PHP_SELF'] <> '/user/log-on.php' => ['redirectUri' => $_SERVER['PHP_SELF']],
            default                                    => []
        };

        self::client()->clear();
        header('Location: ' . self::client()->login($_ENV['AUTH0_BASE_URL'] . '/user/log-on/success', $params));
        exit;
    }

    /**
     * Log off from this application and Auth0
     *
     * @throws ConfigurationException If the Auth0 client is not configured correctly
     */
    public static function logOff(): never
    {
        session_destroy();
        header('Location: ' . self::client()->logout($_ENV['AUTH0_BASE_URL']));
        exit;
    }

    /**
     * Require a user be logged on
     *
     * @param bool $redirect Whether to redirect to log on if there is not a user logged on
     * @return void If it returns, there is a user logged on; if not, we will be redirected to log on
     * @throws ConfigurationException If the Auth0 client is not configured correctly
     */
    public static function requireUser(bool $redirect = true): void
    {
        if (is_null(self::user())) {
            if ($redirect) self::logOn();
            http_response_code(403);
            die('Not Authorized');
        }
    }
}
