<?php
/**
 * @author Daniel J. Summers <daniel@bitbadger.solutions>
 * @license MIT
 */

declare(strict_types=1);

namespace MyPrayerJournal\Domain;

use BitBadger\InspiredByFSharp\Option;
use DateInterval;
use Square\Pjson\{Json, JsonSerialize};

/**
 * The recurrence for a prayer request
 */
class Recurrence
{
    use JsonSerialize;

    /** @var RecurrencePeriod The recurrence period */
    #[Json]
    public RecurrencePeriod $period {
        get => $this->period ?? RecurrencePeriod::Immediate;
        set { $this->period = $value; }
    }

    /** @var int|null How many of the periods will pass before the request is visible again */
    #[Json('interval', omit_empty: true)]
    private ?int $dbInterval {
        get => $this->interval->unwrap();
        set { $this->interval = Option::of($value); }
    }

    /** @var Option<int> How many of the periods will pass before the request is visible again */
    public Option $interval {
        get => $this->interval ?? Option::None();
        set { $this->interval = $value; }
    }

    /**
     * @param RecurrencePeriod $period The recurrence period
     * @param int|null $interval How many of the periods will pass before the request is visible again
     */
    public function __construct(RecurrencePeriod $period, ?int $interval = null)
    {
        $this->period = $period;
        $this->dbInterval = $interval;
    }

    /**
     * Get the date/time interval for this recurrence
     *
     * @return DateInterval The interval matching the recurrence
     */
    public function interval(): DateInterval
    {
        $period = match ($this->period) {
            RecurrencePeriod::Immediate => 'T0S',
            RecurrencePeriod::Hours     => "T{$this->interval->value}H",
            RecurrencePeriod::Days      => "{$this->interval->value}D",
            RecurrencePeriod::Weeks     => ($this->interval->value * 7) . 'D'
        };
        return new DateInterval("P$period");
    }
}
