<?php
/**
 * @author Daniel J. Summers <daniel@bitbadger.solutions>
 * @license MIT
 */

declare(strict_types=1);

namespace MyPrayerJournal\Domain;

use BitBadger\InspiredByFSharp\Option;
use BitBadger\PDODocument\{Custom, DocumentException, DocumentList, Find};
use BitBadger\PDODocument\Mapper\DocumentMapper;
use DateTimeImmutable;
use Exception;
use MyPrayerJournal\Table;
use Square\Pjson\{Json, JsonSerialize};

/**
 * A prayer request
 */
class Request
{
    use JsonSerialize;

    /** @var string The ID for the request */
    #[Json]
    public string $id = '';

    /** @var string The date/time this request was originally entered */
    #[Json]
    public string $enteredOn = '';

    /** @var string The ID of the user to whom this request belongs */
    #[Json]
    public string $userId = '';

    /** @var string|null The date/time the snooze expires for this request (null = not snoozed) */
    #[Json('snoozedUntil', omit_empty: true)]
    private ?string $dbSnoozedUntil {
        get => $this->snoozedUntil->unwrap();
        set { $this->snoozedUntil = Option::of($value); }
    }

    /** @var Option<string> The date/time the snooze expires for this request (None = not snoozed) */
    public Option $snoozedUntil {
        get => $this->snoozedUntil ?? Option::None();
        set { $this->snoozedUntil = $value; }
    }

    /** @var string|null The date/time the current recurrence period is over (null = immediate) */
    #[Json(omit_empty: true)]
    public ?string $showAfter = null;

    /** @var Recurrence The recurrence for this request */
    #[Json]
    public Recurrence $recurrence {
        get => $this->recurrence ?? new Recurrence(RecurrencePeriod::Immediate);
        set => $this->recurrence = $value;
    }

    /** @var History[] The history of this request */
    #[Json(type: History::class)]
    public array $history = [];

    /** @param Note[] $notes Notes regarding this request */
    #[Json(type: Note::class)]
    public array $notes = [];

    /** The current text for this request */
    public string $currentText {
        get => Option::of(array_find($this->history, fn(History $it) => $it->text->isSome))
            ->map(fn(History $it) => $it->text->value)
            ->getOrDefault('');
    }

    /** @var Option<string> The date/time this request was last marked as prayed */
    public Option $lastPrayed {
        get => Option::of(array_find($this->history, fn(History $it) => $it->action === RequestAction::Prayed))
            ->map(fn(History $it) => $it->asOf);
    }

    /** Has this request been answered? */
    public bool $isAnswered {
        get => $this->history[0]->action === RequestAction::Answered;
    }

    /** Is this request currently snoozed? */
    public bool $isSnoozed {
        get => $this->snoozedUntil->isSome
            && new DateTimeImmutable($this->snoozedUntil->value) > new DateTimeImmutable('now');
    }

    /**
     * Is this request currently not shown due to recurrence?
     *
     * @return bool True if the request is pending, false if not
     * @throws Exception If the snoozed or show-after date/times are not valid
     */
    public bool $isPending {
        get => !$this->isSnoozed
            && isset($this->showAfter)
            && new DateTimeImmutable($this->showAfter) > new DateTimeImmutable('now');
    }

    /**
     * Find a request by its ID
     *
     * @param string $id The ID of the request
     * @return Option<Request> The request (if it is found and belongs to the current user)
     * @throws DocumentException If any is encountered
     */
    public static function byId(string $id): Option
    {
        return Find::byId(Table::REQUEST, $id, self::class)
            ->map(fn(Request $it) => $it->userId === $_SESSION['user_id'] ? $it : null);
    }

    /**
     * Get the user's active journal requests
     *
     * @return DocumentList<Request> The requests for the user's journal
     * @throws DocumentException If any is encountered
     */
    public static function forJournal(): DocumentList
    {
        $table = Table::REQUEST;
        return Custom::list(<<<SQL
            SELECT data, (
                   SELECT h.value->>'asOf' as_of
                     FROM $table i LEFT JOIN json_each(i.data, '$.history') h
                    WHERE r.data->>'id' = i.data->>'id' AND h.value->>'action' = 'Prayed'
                    LIMIT 1) last_prayed
              FROM $table r
             WHERE data->>'userId'               = :userId
               AND data->>'$.history[0].action' <> 'Answered'
               AND (data->>'snoozedUntil' IS NULL OR data->>'snoozedUntil' < datetime('now'))
               AND (data->>'showAfter'    IS NULL OR data->>'showAfter'    < datetime('now'))
             ORDER BY coalesce(last_prayed, data->>'snoozedUntil', data->>'showAfter', data->>'$.history[0].asOf')
            SQL, [':userId' => $_SESSION['user_id']], new DocumentMapper(self::class));
    }

    /**
     * Get either the user's active or answered requests
     *
     * @param bool $active True to retrieve active requests, false to retrieve answered requests
     * @param bool $snoozed True to retrieve only snoozed requests
     * @return DocumentList<Request> The requests matching the criteria
     * @throws DocumentException If any is encountered
     */
    private static function forUser(bool $active = true, bool $snoozed = false): DocumentList
    {
        $table = Table::REQUEST;
        $op    = $active ? '<>' : '=';
        $extra = $snoozed ? "AND datetime(data->>'snoozedUntil') > datetime('now')" : '';
        $order = $active
            ? "coalesce(data->>'snoozedUntil', data->>'showAfter', last_prayed, data->>'$.history[0].asOf')"
            : "data->>'$.history[0].asOf' DESC";
        return Custom::list(<<<SQL
            SELECT data, (
                   SELECT h.value->>'asOf' as_of
                     FROM $table i LEFT JOIN json_each(i.data, '$.history') h
                    WHERE r.data->>'id' = i.data->>'id' AND h.value->>'action' = 'Prayed'
                    LIMIT 1) last_prayed
              FROM $table r
             WHERE data->>'userId'                = :userId
               AND data->>'$.history[0].action' $op 'Answered' $extra
             ORDER BY $order
            SQL, [':userId' => $_SESSION['user_id']], new DocumentMapper(self::class));
    }

    /**
     * Get a list of active requests for a user
     *
     * @return DocumentList<Request> The user's active requests
     * @throws DocumentException If any is encountered
     */
    public static function active(): DocumentList
    {
        return self::forUser();
    }

    /**
     * Get a list of answered requests for a user
     *
     * @return DocumentList<Request> The user's answered requests
     * @throws DocumentException If any is encountered
     */
    public static function answered(): DocumentList
    {
        return self::forUser(active: false);
    }

    /**
     * Get a list of snoozed requests for a user
     *
     * @return DocumentList<Request> The user's snoozed requests
     * @throws DocumentException If any is encountered
     */
    public static function snoozed(): DocumentList
    {
        return self::forUser(snoozed: true);
    }
}
