<?php
/**
 * @author Daniel J. Summers <daniel@bitbadger.solutions>
 * @license MIT
 */

declare(strict_types=1);

namespace MyPrayerJournal\UI;

use BitBadger\PDODocument\{DocumentException, DocumentList};
use DateTimeImmutable;
use DateTimeZone;
use Exception;
use MyPrayerJournal\Domain\Request;

/**
 * User interface building blocks
 */
class Component
{
    /**
     * Generate a material icon
     *
     * @param string $name The name of the material icon
     * @return string The material icon wrapped in a `span` tag
     */
    public static function icon(string $name): string {
        return "<span class=material-icons>$name</span>";
    }

    /**
     * Render the journal items for the current user
     *
     * @throws DocumentException If any is encountered
     */
    public static function journal(): void
    {
        Layout::bareHead();
        $reqs = Request::forJournal();
        if ($reqs->hasItems) { ?>
            <section id=journalItems class="row row-cols-1 row-cols-md-2 row-cols-lg-3 row-cols-xl-4 g-3" hx-target=this
                     hx-swap=outerHTML aria-label="Prayer Requests"><?php
            $spacer = '<span>&nbsp;</span>';
            foreach ($reqs->items as /** @var Request $req */ $req) {
                $lastPrayed   = $req->lastPrayed;
                $lastActivity = $lastPrayed->getOrDefault($req->history[0]->asOf); ?>
                <div class=col>
                <div class="card h-100">
                    <div class="card-header p-0 d-flex" role=toolbar>
                        <?=self::pageLink("/request/edit?id=$req->id", self::icon('edit'),
                            ['class' => 'btn btn-secondary', 'title' => 'Edit Request'])?>
                        <?=$spacer?>
                        <button type=button class="btn btn-secondary" title="Add Notes" data-bs-toggle=modal
                                data-bs-target=#notesModal hx-get="/components/request/add-note?id=<?=$req->id?>"
                                hx-target=#notesBody hx-swap=innerHTML><?=self::icon('comment');?></button>
                        <?=$spacer?>
                        <button type=button class="btn btn-secondary" title="Snooze Request" data-bs-toggle=modal
                                data-bs-target=#snoozeModal hx-get="/components/request/snooze?id=<?=$req->id?>"
                                hx-target=#snoozeBody hx-swap=innerHTML><?=self::icon('schedule');?></button>
                        <div class=flex-grow-1></div>
                        <button type=button class="btn btn-success w-25" hx-patch="/request/prayed?id=<?=$req->id?>"
                                title="Mark as Prayed"><?=self::icon('done');?></button>
                    </div>
                    <div class=card-body>
                        <p class=request-text><?=htmlentities($req->currentText);?>
                    </div>
                    <div class="card-footer text-end text-muted px-1 py-0">
                        <em>last <?=$lastPrayed->map(fn() => 'prayed')->getOrDefault('activity')?>
                            <?=self::relativeDate($lastPrayed->getOrDefault($req->history[0]->asOf))?>
                        </em>
                    </div>
                </div>
                </div><?php
            } ?>
            </section><?php
        } else {
            Component::noResults('No Active Requests', '/request/edit?id=new', 'Add a Request', <<<'TEXT'
                You have no requests to be shown; see the &ldquo;Active&rdquo; link above for snoozed or deferred
                requests, and the &ldquo;Answered&rdquo; link for answered requests
                TEXT);
        }
        Layout::bareFoot();
    }

    /**
     * Create a card when there are no results found
     */
    public static function noResults(string $heading, string $link, string $buttonText, string $text): void
    { ?>
        <div class=card>
            <h5 class=card-header><?=$heading?></h5>
            <div class="card-body text-center">
                <p class=card-text><?=$text?></p>
                <?=self::pageLink($link, $buttonText, ['class' => 'btn btn-primary'])?>
            </div>
        </div><?php
    }

    /**
     * Create a link to a page within myPrayerJournal
     *
     * @param string $href The URL for the link
     * @param string $text The text for the link
     * @param array $attrs Any additional attributes that should be placed on the `a` tag
     */
    public static function pageLink(string $href, string $text, array $attrs = []): string
    {
        $extraAttrs = array_reduce(array_keys($attrs),
            fn($acc, $key) => $acc . sprintf(' %s="%s"', $key, $attrs[$key]), '');
        return sprintf('<a href="%s" hx-get="%s" hx-target=#top hx-swap=innerHTML hx-push-url=true%s>%s</a>', $href,
            $href, $extraAttrs, $text);
    }

    /**
     * Create a relative date with a tooltip with the actual date/time
     *
     * @return string The HTML for the relative date
     * @throws Exception If the date/time cannot be parsed
     */
    public static function relativeDate(string $date): string
    {
        $parsed = new DateTimeImmutable($date);
        $iso    = $parsed->format('c');
        $title  = $parsed->setTimezone(new DateTimeZone($_REQUEST['time_zone']))->format('l, F j, Y \a\t g:ia T');
        return "<relative-date-time title=\"$title\" interval=10000>$iso</relative-date-time>";
    }

    /**
     * Render a request list item
     *
     * @param Request $req The request on which the list items should be based
     * @throws Exception If date/time values cannot be parsed
     */
    public static function requestItem(Request $req): void
    {
        $btnClass   = "btn btn-light mx-2";
        $restoreBtn = fn(string $id, string $link, string $title) => sprintf(
            '<button class="%s" hx-patch="/request/%s?id=%s" title="%s" hx-target=#req-%s hx-swap=outerHTML>%s</button>',
            $btnClass, $link, $id, htmlspecialchars($title), $id, self::icon('restore')); ?>
        <div class="list-group-item px-0 d-flex flex-row align-items-start" id=req-<?=$req->id?>><?php
            echo self::pageLink("/request/full?id=$req->id", self::icon('description'),
                    ['class' => $btnClass, 'title' => 'View Full Request']);
            if (!$req->isAnswered) {
                echo self::pageLink("/request/edit?id=$req->id", self::icon('edit'),
                        ['class' => $btnClass, 'title' => 'Edit Request']);
            }
            if ($req->isSnoozed) {
                echo $restoreBtn($req->id, 'cancel-snooze', 'Cancel Snooze');
            } elseif ($req->isPending) {
                echo $restoreBtn($req->id, 'show', 'Show Now');
            }
            echo '<p class="request-text mb-0">' . htmlentities($req->currentText);
            if ($req->isSnoozed || $req->isPending || $req->isAnswered) { ?>
                <br>
                <small class=text-muted><em><?php
                    echo match (true) {
                        $req->isSnoozed => 'Snooze expires ' . self::relativeDate($req->snoozedUntil->value),
                        $req->isPending => 'Request appears next ' . self::relativeDate($req->showAfter),
                        default         => 'Answered ' . self::relativeDate($req->history[0]->asOf)
                    };?>
                </em></small><?php
            } ?>
        </div><?php
    }

    /**
     * Render the given list of requests
     *
     * @param DocumentList<Request> $reqs The list of requests to render
     * @throws Exception If date/time instances are not valid
     */
    public static function requestList(DocumentList $reqs): void
    {
        echo '<div class=list-group>';
        $reqs->iter(self::requestItem(...));
        echo '</div>';
    }
}
