<?php
/**
 * @author Daniel J. Summers <daniel@bitbadger.solutions>
 * @license MIT
 */

declare(strict_types=1);

namespace MyPrayerJournal\UI;

use BitBadger\PDODocument\Custom;
use BitBadger\PDODocument\Mapper\ExistsMapper;
use MyPrayerJournal\Table;

class Layout
{
    /**
     * Generate the heading for a bare result
     */
    public static function bareHead(): void
    {
        echo '<!DOCTYPE html><html lang=en><head><title></title></head><body>';
    }

    /**
     * Generate the end of a bare result
     */
    public static function bareFoot(): void
    {
        echo '</body></html>';
    }

    /**
     * Is this an htmx request?
     *
     * @return bool True if this is an htmx request, false if not
     */
    private static function isHtmx(): bool
    {
        return key_exists('HTTP_HX_REQUEST', $_SERVER) && !key_exists('HTTP_HX_HISTORY_RESTORE_REQUEST', $_SERVER);
    }

    /**
     * Create the `DOCTYPE` declaration, `html`, and `head` tags for the page
     *
     * @param string $title The title of the page
     */
    public static function htmlHead(string $title): void
    {
        if (self::isHtmx()) {
            echo "<!DOCTYPE html><html lang=en><head lang=en><title>$title &#xab; myPrayerJournal</title></head>";
        } else {
            echo <<<HEAD
                <!DOCTYPE html>
                <html lang=en>
                <head>
                    <meta name=viewport content="width=device-width, initial-scale=1">
                    <meta name=description content="Online prayer journal - free w/Google or Microsoft account">
                    <meta name=htmx-config content='{"historyCacheSize":0}'>
                    <title>$title &#xab; myPrayerJournal</title>
                    <link href=https://cdn.jsdelivr.net/npm/bootstrap@5.3.2/dist/css/bootstrap.min.css rel=stylesheet
                          integrity="sha384-T3c6CoIi6uLrA9TneNEoa7RxnatzjcDSCmG1MXxSR1GAsXEV/Dwwykc2MPK8M2HN"
                          crossorigin=anonymous>
                    <link href="https://fonts.googleapis.com/icon?family=Material+Icons" rel=stylesheet>
                    <link href=/_/style.css rel=stylesheet>
                </head>
                HEAD;
        }
    }

    private static function navLink(string $url, string $text): void
    {
        $classAttr = match (true) {
            str_starts_with($_SERVER['PHP_SELF'], $url) => ['class' => 'is-active-route'],
            default => []
        };
        echo '<li class=nav-item>' . Component::pageLink($url, $text, $classAttr);
    }

    /**
     * The default navigation bar, which will load the items on page load, and whenever a refresh event occurs
     */
    public static function navBar(): void
    {
        $table      = Table::REQUEST;
        $hasSnoozed = key_exists('user_id', $_SESSION)
            ? Custom::scalar(<<<SQL
                SELECT EXISTS (
                    SELECT 1 FROM $table
                     WHERE data->>'userId' = :userId AND datetime(data->>'snoozedUntil') > datetime('now'))
                SQL, [':userId' => $_SESSION['user_id']], new ExistsMapper())
            : false; ?>
        <nav class="navbar navbar-dark" role="navigation">
            <div class=container-fluid>
                <?=Component::pageLink('/',
                    '<span class=m>my</span><span class=p>Prayer</span><span class=j>Journal</span>',
                    ['class' => 'navbar-brand'])?>
                <ul class="navbar-nav me-auto d-flex flex-row"><?php
                    if (key_exists('user_id', $_SESSION)) {
                        self::navLink('/journal', 'Journal');
                        self::navLink('/requests/active', 'Active');
                    if ($hasSnoozed) self::navLink('/requests/snoozed', 'Snoozed');
                        self::navLink('/requests/answered', 'Answered'); ?>
                        <li class=nav-item><a href=/user/log-off>Log Off</a><?php
                    } else { ?>
                        <li class=nav-item><a href=/user/log-on>Log On</a><?php
                    }
                    self::navLink('/docs', 'Docs'); ?>
                </ul>
            </div>
        </nav><?php
    }

    /**
     * Drop .0 or .0.0 from the end of the version to format it for display
     *
     * @return string The version of the application for user display
     */
    private static function displayVersion(): string {
        [$major, $minor, $rev] = explode('.', MPJ_VERSION);
        $minor = $minor == '0' ? '' : ".$minor";
        $rev = match (true) {
            $rev == '0'                 => '',
            str_starts_with($rev, '0-') => substr($rev, 1),
            default                     => ".$rev"
        };
        return "v$major$minor$rev";
    }

    /**
     * Create the footer
     */
    public static function htmlFoot(): void
    { ?>
        <footer class=container-fluid>
            <p class="text-muted text-end">
                myPrayerJournal <?=self::displayVersion();?><br>
                <em><small>
                    <?=Component::pageLink('/legal/privacy-policy', 'Privacy Policy')?> &bull;
                    <?=Component::pageLink('/legal/terms-of-service', 'Terms of Service')?> &bull;
                    <a href=https://git.bitbadger.solutions/bit-badger/myPrayerJournal target=_blank
                       rel=noopener>Developed</a> and hosted by
                    <a href=https://bitbadger.solutions target=_blank rel=noopener>Bit Badger Solutions</a>
                </small></em>
            <script src=https://unpkg.com/htmx.org@2.0.4 crossorigin=anonymous
                    integrity="sha384-HGfztofotfshcF7+8n44JQL2oJmowVChPTg48S+jvZoztPfvwD79OC/LTtG6dMp+"></script>
            <script>if (!htmx) document.write('<script src=\"/_/htmx.min.js\"><\/script>')</script>
            <script async src=https://cdn.jsdelivr.net/npm/bootstrap@5.3.2/dist/js/bootstrap.bundle.min.js
                    integrity="sha384-C6RzsynM9kWDrMNeT87bh95OGNyZPhcTNXj1NW7RuBCsyN/o0jlpcV8Qyq46cDfL"
                    crossorigin=anonymous></script>
            <script>
                setTimeout(function () {
                    if (!bootstrap) document.write('<script src=\"/_/bootstrap.bundle.min.js\"><\/script>')
                }, 2000)
            </script>
            <script src=/_/mpj.js></script>
            <script src=/_/relative-date-time.js defer></script>
        </footer><?php
    }

    /**
     * Generate the heading for a full or partial page result
     *
     * @param string $title The title of the page
     */
    public static function pageHead(string $title): void
    {
        self::htmlHead($title);
        echo '<body>';
        if (!self::isHtmx()) echo '<section id=top aria-label="Top navigation">';
        self::navBar();
        echo '<main role=main>';
    }

    /**
     * Generate the end of the page for a full or partial page result
     */
    public static function pageFoot(): void
    {
        echo '</main>';
        if (!self::isHtmx()) {
            echo '</section>';
            self::htmlFoot();
        }
        echo '</body></html>';
    }
}
