<?php
/**
 * @author Daniel J. Summers <daniel@bitbadger.solutions>
 * @license MIT
 */

declare(strict_types=1);

use Auth0\SDK\Exception\ConfigurationException;
use BitBadger\PDODocument\{AutoId, Configuration, Definition, DocumentException};
use Dotenv\Dotenv;
use MyPrayerJournal\{Auth, Table};
use MyPrayerJournal\Domain\Request;

require __DIR__ . '/vendor/autoload.php';

/** The version of this application */
const MPJ_VERSION = '4.0.0-beta2';

(Dotenv::createImmutable(__DIR__))->load();

if (php_sapi_name() != 'cli') {
    session_start();

    $auth0_user = Auth::user();
    if (!is_null($auth0_user)) {
        $_SESSION['user_id'] = $auth0_user['sub'];
    }

    $_REQUEST['time_zone'] = $_SERVER['HTTP_X_TIME_ZONE'] ?? 'Etc/UTC';
}

Configuration::useDSN('sqlite:' . implode(DIRECTORY_SEPARATOR, [__DIR__, 'data', 'mpj.db']));
Configuration::$autoId         = AutoId::RandomString;
Configuration::$idStringLength = 12;

Definition::ensureTable(Table::REQUEST);
Definition::ensureFieldIndex(Table::REQUEST, 'user', ['userId']);

$_PATCH = [];
if ($_SERVER['REQUEST_METHOD'] ?? '' === 'PATCH') parse_str(file_get_contents('php://input'), $_PATCH);

/**
 * Return a 404 and exit
 */
function not_found(): never
{
    http_response_code(404);
    die('Not found');
}

/**
 * Return a 303 redirect ("see other" - redirects to a GET)
 *
 * @param string $url The URL to which the browser should be redirected
 */
function see_other(string $url): never
{
    header('Location: ' . (str_starts_with($url, 'http') ? '/' : $url));
    http_response_code(303);
    exit;
}

/**
 * Add a header that instructs the browser to close an open modal dialog
 *
 * @param string $name The name of the dialog to be closed
 */
function hide_modal(string $name): void
{
    header("X-Hide-Modal: $name");
}

/**
 * Validate the user, HTTP method, and request
 *
 * @param string $id The ID of the prayer request to retrieve
 * @param array $methods The allowable HTTP methods
 * @param bool $redirect Whether to redirect not-logged-on users (optional, defaults to true)
 * @return Request The request (failures will not return)
 * @throws ConfigurationException|DocumentException If any is encountered
 */
function validate_request(string $id, array $methods, bool $redirect = true): Request
{
    if (empty(array_filter($methods, fn($it) => $_SERVER['REQUEST_METHOD'] === $it))) not_found();

    Auth::requireUser($redirect);

    return Request::byId($id)->getOrCall(not_found(...));
}
